@@ -2,6 +2,8 @@ package Quiki;
 
 use feature ':5.10';
 
+use Data::Dumper;
+
 use Quiki::Formatter;
 use Quiki::Meta;
 use Quiki::Users;
@@ -10,9 +12,9 @@ use Quiki::Pages;
 use CGI qw/:standard *div/;
 use CGI::Session;
 use HTML::Template::Pro;
-use Gravatar::URL;
 use File::MMagic;
 use File::Slurp 'slurp';
+use Pod::Html;
 
 use warnings;
 use strict;
@@ -23,11 +25,11 @@ Quiki - A lightweight Wiki in Perl
 
 =head1 VERSION
 
-Version 0.01
+Version 0.02
 
 =cut
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 
 =head1 SYNOPSIS
@@ -45,7 +47,7 @@ Creates a new Quiki object.
 
 =head2 run
 
-Runs de Quiki.
+Runs the Quiki.
 
 =cut
 
@@ -67,13 +69,14 @@ sub new {
     $self->{DOCROOT} = $ENV{SCRIPT_NAME};
     $self->{DOCROOT} =~ s!/[^/]+$!/!;
 
+    $self->{QUIKI_ID} = $self->{name} . '_' . slurp 'data/quiki_id';
     return bless $self, $class;
 }
 
 sub run {
     my $self = shift;
 
-    $self->{sid} = cookie("QuikiSID") || undef;
+    $self->{sid} = cookie($self->{QUIKI_ID}) || undef;
     $self->{session} = new CGI::Session(undef, $self->{sid}, undef);
 
     # XXX
@@ -81,11 +84,34 @@ sub run {
     my $action = param('action') || '';
 
     # XXX -- temos de proteger mais coisas, possivelmente
+    #        E era giro termos filename e displayname
     $node =~ s/\s/_/g;
 
     $self->{meta} = Quiki::Meta::get($node);
     $self->{node} = $node;
 
+    if ($action eq 'update_perms') {
+        my $username = param('edit_user');
+        if (param("admin_action") eq "Delete") {
+            if ($username eq "admin") {
+                $self->{session}->param('msg', "Admin account can not be deleted.");
+            } else {
+                Quiki::Users->delete($username);
+                $self->{session}->param('msg', "User '$username' deleted.");
+            }
+        }
+        if (param("admin_action") eq "Save") {
+            my $perm = param("perms");
+            if ($username eq "admin") {
+                $self->{session}->param('msg', "Admin account righs can not be changed.");
+            } else {
+                Quiki::Users->update($username, perm_group => $perm);
+                $self->{session}->param('msg', "Permission rights changed for user '$username'.");
+            }
+        }
+        $action = 'admin_page';
+    }
+
     if ($action eq 'save_profile' && param('submit') =~ /^Save/) {
         if (param("new_password1") && (param("new_password1") ne param("new_password2"))) {
             $self->{session}->param('msg', "Passwords do not match. Try again!");
@@ -107,7 +133,7 @@ sub run {
             if (param("filename$_") && param("name$_")) {
                 my $id = param("name$_");
                 my $path = "data/attach/$node";
-                (! -f $path) and mkdir $path and chown 0777, $path;
+                -f $path or (mkdir $path and chown 0777, $path);
                 open OUT, ">", "$path/$id" or die "Can't create out file: $!";
                 my $filename = param("filename$_");
                 my ($buffer, $bytesread);
@@ -213,7 +239,7 @@ sub run {
     }
 
 
-    my $cookie = cookie('QuikiSID' => $self->{session}->id);
+    my $cookie = cookie($self->{QUIKI_ID} => $self->{session}->id);
     print header(-charset=>'UTF-8',-cookie=>$cookie);
 
     my @trace;
@@ -238,7 +264,9 @@ sub run {
 
     my $username = ($self->{session}->param('authenticated')?
                     $self->{session}->param('username'):"guest");
+    ## XXX - Later, join these functions to query database only once
     my $email    = Quiki::Users->email($username);
+    my $urole    = Quiki::Users->role($username);
     my $theme    = $self->{theme} || 'default';
 
     my $template = HTML::Template::Pro->new(filename => "themes/$theme/wrapper.tmpl",
@@ -253,13 +281,16 @@ sub run {
                      LAST_REV    => (($self->{rev} || 0) == ($self->{meta}{rev} || 0)),
                      REV         => $self->{rev},
                      BREADCUMBS  => $breadcumbs,
+                     SERVERNAME  => $self->{SERVER_NAME},
                      DOCROOT     => $self->{DOCROOT},
+                     OPENSITE    => $self->{opensite} // 1,
+                     USER_ROLE   => $urole,
                      PREVIEW     => $preview,
                     );
 
     if ($action eq 'profile_page') {
         $template->param(EMAIL       => $email,
-                         GRAVATAR    => gravatar_url(email => $email));
+                         GRAVATAR    => Quiki::Users->gravatar($username));
     }
 
     if ($action eq 'edit' && 
@@ -315,19 +346,23 @@ sub run {
             if ($i != $self->{meta}{rev}) {
                 $entry->{AUTHOR} = $self->{meta}{revs}{$i}{last_update_by};
                 $entry->{DATE} =  $self->{meta}{revs}{$i}{last_updated_in};
-                $entry->{GRAVATAR} = gravatar_url(email => Quiki::Users->email($self->{meta}{revs}{$i}{last_update_by}));
+                $entry->{GRAVATAR} = Quiki::Users->gravatar($self->{meta}{revs}{$i}{last_update_by});
             }
             else {
                 $entry->{AUTHOR} = $self->{meta}{last_update_by};
                 $entry->{DATE} =  $self->{meta}{last_updated_in};
-                $entry->{GRAVATAR} = gravatar_url(email => Quiki::Users->email($self->{meta}{last_update_by}));
+                $entry->{GRAVATAR} = Quiki::Users->gravatar($self->{meta}{last_update_by});
             }
             push @revs, $entry;
         }
         $template->param(REVISIONS => \@revs);
     }
+    elsif ($action eq 'admin_page') {
+        my $users = Quiki::Users->list;
+        $template->param(USERS => $users);
+    }
     elsif ($action eq 'index') {
-        opendir(DIR,'data/content/');
+        opendir DIR, 'data/content/';
         my @pages;
         for my $f (sort { lc($a) cmp lc($b) } readdir(DIR)) {
             unless ($f=~/^\./) {
@@ -337,11 +372,11 @@ sub run {
                     NAME => $f,
                     AUTHOR => $meta->{last_update_by},
                     DATE => $meta->{last_updated_in},
-                    GRAVATAR => gravatar_url(email => Quiki::Users->email($meta->{last_update_by})),
+                    GRAVATAR => Quiki::Users->gravatar($meta->{last_update_by}),
                   }
               }
         }
-        closedir(DIR);
+        closedir DIR;
         $template->param(PAGES=>\@pages);
     }
     elsif ($action eq 'diff') {
@@ -364,7 +399,7 @@ sub run {
     unless ($action eq 'edit') {
         my $L_META;
         if ($self->{meta}{last_update_by}) {
-            my $url = gravatar_url(email => Quiki::Users->email($self->{meta}{last_update_by}));
+            my $url = Quiki::Users->gravatar($self->{meta}{last_update_by});
             $L_META = img({-src => $url, -width => '24', -style => 'vertical-align: middle'});
             $L_META .= sprintf("&nbsp;Last edited by %s, in %s",
                                $self->{meta}{last_update_by},
@@ -389,7 +424,47 @@ sub run {
     $template->output(print_to => \*STDOUT);
 }
 
-=head1 SYNTAX
+=head1 QUIKI CONFIGURATION FILE
+
+After a Quiki wiki is deployed with C<< quiki_create >> a
+C<< quiki.conf >> file can be edited to configure Quiki behavior. While
+later versions might offer a web interface to configure them, at
+present you need to use a text editor and change the file.  Note that
+it is a Perl file. Therefore it should parse correctly by Perl. You
+can check it using C<< perl -c quiki.conf >>.
+
+Quiki configuration file supports the following keys:
+
+=over 4
+
+=item name
+
+The display name for your wiki
+
+=item theme
+
+The theme to be used. Note that at the time we are writing this only
+the C<< default >> theme exists
+
+=item index
+
+The name of the main quiki page. It defaults to C<< index >>.
+
+=item opensite
+
+This is a boolean value. By default it is true (1), meaning the
+register button will be available to everybody visiting your
+Quiki. Turn it off setting it to false (0).
+
+At the current moment there isn't any other way to register. Therefore
+you should turn registering off only after all users have an account,
+or you need to turn it off manually everytime a new user has to
+register.
+
+=back
+
+
+=head1 QUIKI SYNTAX
 
 Quiki wiki syntax is very similar to other wiki, and especially
 similar with dokuwiki syntax.
@@ -607,7 +682,7 @@ L<http://search.cpan.org/dist/Quiki/>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009 Alberto Simoes and Nuno Carvalho.
+Copyright 2009-2010 Alberto Simoes and Nuno Carvalho.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
